﻿module WordAndExcel

open Microsoft.Office.Interop.Excel

// Run Excel 
let app = new ApplicationClass(Visible = true) 

// Create a new file, and get the 1st worksheet
let workbook = app.Workbooks.Add(XlWBATemplate.xlWBATWorksheet) 

// Note that worksheets are started from 1 not 0
let worksheet = (workbook.Worksheets.[1] :?> Worksheet)

// define the type provider
type Demographics = Microsoft.FSharp.Data.TypeProviders.ODataService<ServiceUri = 
    "https://api.datamarket.azure.com/Uk.Gov/TravelAdvisoryService/">
let ctx = Demographics.GetDataContext()

// set the credentials
// ctx.Credentials <- System.Net.NetworkCredential(<liveID>, <id>)

// declare a NewsRecord class
type NewsRecord() = 
    member val Title = System.String.Empty with get, set
    member val Summary = System.String.Empty with get, set

// query the latest news
let latestNews = query {
    for n in ctx.LatestTravelNewsFromFco do
    select (NewsRecord(Title = n.Title, Summary = n.Summary))
}

let news = latestNews |> Seq.toArray

// get the range tuple such as "A1", "A10"
let getRangeStrings columnName news = 
    let start = sprintf "%c%d" columnName 1
    let ``end`` = sprintf "%c%d" columnName (news |> Seq.length)
    start, ``end``

let len = news.Length

// get range tuple, which contains start and end
let range0 = (getRangeStrings 'B' news)
let range1 = (getRangeStrings 'C' news)

// insert title to B column
(worksheet.Range (fst range0, snd range0)).Value2 <- Array2D.init len 1 (fun x _ -> news.[x].Title)

// insert summary to C column
(worksheet.Range (fst range1, snd range1)).Value2 <- Array2D.init len 1 (fun x _ -> news.[x].Summary)


